﻿using System;
using System.Collections.Generic;


namespace cristian_docentes
{
    /*
            CRISTIAN_DOCENTES

            Aplicativo de consola para mantenimiento de una base de datos de dominio propio.

    */
    class Program
    {
        public static bool          loop        = true;                 /* Bandera de control del bucle iterativo infinito */
        public static database      db          = new database();       /* Clase gestora del mantenimiento de datos */

        #region Main
        /// <summary>
        /// 
        /// Punto de entrada en la aplicación
        /// 
        /// Crea un bucle infinito que ejecuta la función MENU() y se mantiene en la iteración por tiempo ilimitado
        /// hasta que el usuario presiona la tecla -ESCAPE-, momento en el que la bandera LOOP cambia su valor a false
        /// el flujo abandona el bucle y se ejecutan las instrucciones siguientes.
        /// 
        /// a.- Borrado del contenido de la pantalla de consola.
        /// b.- Llamada al destructor de la clase <database> encargada de actualizar datos y cerrar ficheros.
        /// 
        /// </summary>
        public static void Main()
        {
            do { menu(); } while (loop);

            Console.Clear();
            db.Dispose();
        }

        /// <summary>
        /// menu()
        /// 
        /// Función que muestra las diferentes opciones de menú y se mantiene a la escucha 
        /// de pulsaciones del teclado a la espera de la decisión del usuario.
        /// 
        /// Esta función ignora cualquier tecla que no represente ninguna opción de menú o la orden de
        /// cerrar la aplicación actualizando los datos residentes en memoria.
        /// 
        /// </summary>
        public static void menu()
        {
            Console.Title = "Mantenimiento Docentes";
            Console.Clear();

            Console.CursorLeft = 10;    Console.WriteLine("Opciones de menú");
            Console.CursorLeft = 10;    Console.WriteLine("----------------");
            Console.CursorLeft = 10;    Console.WriteLine("1.- Añadir registro.");
            Console.CursorLeft = 10;    Console.WriteLine("2.- Eliminar registro.");
            Console.CursorLeft = 10;    Console.WriteLine("3.- Eliminar todos los registros.");
            Console.CursorLeft = 10;    Console.WriteLine("4.- Listar contenido.");
            Console.CursorLeft = 10;    Console.WriteLine("5.- Imprimir contenido.");
            Console.CursorLeft = 10;    Console.WriteLine("6.- Actualizar datos y continuar.");
            Console.CursorLeft = 10;    Console.WriteLine("");
            Console.CursorLeft = 10;    Console.Write("Pulse opción de menú / Tecla -Escape- para salir de la aplicación.");

            switch (Console.ReadKey(true).Key)
            {
                case ConsoleKey.NumPad1:
                    db.addDocente(getDocente());
                    break;
                case ConsoleKey.NumPad2:
                    db.removeDocente();
                    break;
                case ConsoleKey.NumPad3:
                    db.RemoveAll();
                    break;
                case ConsoleKey.NumPad4:
                    db.Lista();
                    break;
                case ConsoleKey.NumPad5:
                    db.Imprime();
                    break;
                case ConsoleKey.NumPad6:
                    db.closeDatabase();
                    db.openDatabase();
                    break;
                case ConsoleKey.Escape:
                    loop = false;
                    break;
            }
        }
        #endregion

        #region Funciones de gestión de información
        /// <summary>
        /// getDocente()
        /// 
        /// Esta función devuelve una implementación de la clase docente con los datos que el usuario informa con el telado.
        /// 
        /// Muestra en la consola los nombres de los campos a introducir, almacenando la posición vertical en la consola de cada uno de ellos
        /// para hacer más elegante, desde el punto de vista visual, la introducción de la información.
        /// 
        /// Enumera y muestra la lista <docentes.licencia> para gestionar el literal de la categoría del docente según la opción tecleada por el usuario.
        /// 
        /// Cuenta con el auxilio de las funciones encargadas de verificar que la información del usuario corresponde a un dato correcto.
        /// 
        /// 
        ///     readInt()           Verifica los datos del usuario y devuelve un objeto tipo <int>, no permitiendo datos no numéricos o aquellos que se encuentren
        ///                         fuera de los límites establecios.
        ///     
        ///     readString()        Devuelve una cadena con la totalidad de caracteres tecleados por el usuario.
        ///     
        ///     readCategoria()     Devuelve el literal la titulación asociada al docente, no permitiendo que el usuario introduzca un valor que no exista
        ///                         en la enumeración docente.licencia. Como excepción, si el usuario pulsa la tecla -Escape- se cancelará la actualización
        ///                         del registro en el fichero de datos. (
        ///                         
        /// 
        /// </summary>
        /// <returns>Clase docente con los datos introducidos por el usuario.</returns>
        public static docentes getDocente()
        {
            docentes item = new docentes();
            int[] posY = new int[3];
            int yPos = 0;
            Console.Clear();

            Console.WriteLine("Nuevo docente.");
            Console.WriteLine("................................");
            Console.WriteLine();
            yPos = Console.CursorTop;

            posY[0] = Console.CursorTop;    Console.WriteLine("Ident:");
            posY[1] = Console.CursorTop;    Console.WriteLine("Nombre: ");
            posY[2] = Console.CursorTop;    Console.WriteLine("Categoría: ");
            Console.WriteLine();
            Console.WriteLine();
            Console.WriteLine("**  Categorías  **");
            Console.WriteLine();
            
            /* Muestra una lista con los nombres de la enumeración recogida en docentes.licencia con su correspondiente valos numérico */
            int index = 0;
            foreach(string cat in Enum.GetNames(typeof (docentes.licencia)))
            {
                ++index;                                                /* Incrementamos el número ordinal de cada item de la enumeración docentes.licencia */
                Console.WriteLine(index.ToString() + ".-  " + cat);
            }
            Console.WriteLine();
            Console.WriteLine("Escape en categoría cancela el registro de este dato.");

            item.ident      = readInt(posY[0], 25, 1, int.MaxValue);
            item.nombre     = readString(posY[1], 25);
            item.categoria  = readCategoria(posY[2], 25);


            return item;
        }

        public static int readInt(int y, int x, int min, int max)
        {
            int ret = 0;
            bool control = true;
            do
            {
                Console.CursorTop = y; Console.CursorLeft = x; Console.Write("                      ");
                Console.CursorTop = y; Console.CursorLeft = x;
                string keyboard = Console.ReadLine();
                try
                {
                    ret = int.Parse(keyboard);
                    if (ret >= min && ret <= max) control = false;
                }
                catch { }
            } while (control == true);
            return ret;
        }
        public static string readString(int y, int x)
        {
            string ret = "";
            Console.CursorTop = y; Console.CursorLeft = x;
            ret = Console.ReadLine();
            return ret;
        }
        public static string readCategoria(int y, int x)
        {
            string ret = "";
            bool control = true;
            //Console.WriteLine("Categoría Docente");
            do
            {
                Console.CursorLeft = x;
                Console.CursorTop = y;
                switch (Console.ReadKey(true).Key)
                {
                    case ConsoleKey.NumPad1:
                        ret = Enum.GetName(typeof(docentes.licencia), 1);
                        Console.Write(ret);
                        control = false;
                        break;
                    case ConsoleKey.NumPad2:
                        ret = Enum.GetName(typeof(docentes.licencia), 2);
                        Console.Write(ret);
                        control = false;
                        break;
                    case ConsoleKey.NumPad3:
                        ret = Enum.GetName(typeof(docentes.licencia), 3);
                        Console.Write(ret);
                        control = false;
                        break;
                    case ConsoleKey.Escape:
                        Console.Write("*CANCELADO*");
                        control = false;
                        break;

                }

            } while (control == true);

            return ret;
        }
        #endregion
    }
}
