﻿using System;
using System.Collections.Generic;

using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

using System.Windows.Forms;
using System.Drawing.Printing;


namespace cristian_docentes
{
    public class database
    {
        protected SortedDictionary<string, docentes> bTree      = new SortedDictionary<string, docentes>();

        protected string appPath                                = Application.StartupPath;
        protected string dbName                                 = "database_docentes.cristian";
        protected string incidencia                             = "ok";

        public SortedDictionary<string, docentes> registros { get { return bTree; } }

        public database()
        {
            openDatabase();
        }
        ~database() { Dispose(); }
        public void Dispose()
        {
            closeDatabase();
        }

        /// <summary>
        /// Inserta un nuevo registro de docente en la base de datos, verificando previamente si su nombre ya existe.
        /// En caso de existir la aplicación preguntará al ususario si desea actualizar el registro existente con los
        /// datos introducidos.
        /// 
        /// La comprobación de existencia de dato se realiza invocando la función CONTAINSKEY de la clase SortedDictionary con
        /// el nombre del docente transformado a mayúsculas para evitar errores conceptuales debidos a diferente tratamiento de
        /// las mayúsculas-minúsculas. Las claves siempre se transforman en mayúsculas para la gestion en los índices del fichero.
        /// </summary>
        /// <param name="item"></param>
        public void addDocente(docentes item)
        {
            if (item.categoria != "")
            {
                string key = item.nombre.ToUpper();
                if (bTree.ContainsKey(key))
                {
                    if (MessageBox.Show("Nombre_Docente ya existe en la base de datos\r\nDesea actualizarlo ? ", "Duplicado", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        bTree[key] = item;
                    }
                }
                else
                {
                    bTree.Add(key, item);
                }
            }
        }

        /// <summary>
        /// removeDocente()
        /// Solicita el nombre del docente que se desea eliminar, verifica si existe y lo elimina previa advertencia al ususario.
        /// </summary>
        public void removeDocente()
        {
            Console.Clear();
            Console.Write("Nombre a eliminar: ");
            string key = Console.ReadLine().ToUpper();
            if (bTree.ContainsKey(key))
            {
                if (MessageBox.Show("Está completamente seguro de eliminar registro ?", "Eliminar", MessageBoxButtons.YesNo ) == DialogResult.Yes)
                {
                    bTree.Remove(key);
                }
            }
            else
            {
                MessageBox.Show("Nombre_Docente no existe en la base de datos\r\n", "Eliminar", MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Lista()
        /// Lista todos los registros de la base de datos.
        /// </summary>
        public void Lista()
        {
            Console.Clear();
            Console.CursorLeft = 10 - ("Ident").Length; Console.Write("IDENT");
            Console.CursorLeft = 15; Console.Write("NOMBRE");
            Console.CursorLeft = 70; Console.WriteLine("CATEGORIA");
            Console.CursorLeft = 10 - ("Ident").Length; Console.Write("-----");
            Console.CursorLeft = 15; Console.Write("------");
            Console.CursorLeft = 70; Console.WriteLine("---------");

            foreach (KeyValuePair<string, docentes> d in bTree)
            {
                Console.CursorLeft = 10 - d.Value.ident.ToString("#,##0").Length; Console.Write(d.Value.ident.ToString("#,##0"));
                Console.CursorLeft = 15; Console.Write(d.Value.nombre);
                Console.CursorLeft = 70; Console.WriteLine(d.Value.categoria);
            }
            Console.WriteLine();
            Console.CursorLeft = 15;    Console.Write("Pulse la tecla -Escape- para volver al menú.");
            while (Console.ReadKey(true).Key != ConsoleKey.Escape) ;
            Console.Clear();
        }

        public void Imprime()
        {
            printer impreso = new printer();
        }

        /// <summary>
        /// RemoveAll()
        /// 
        /// Elimina todos los datos del fichero.
        /// Pregunta al usuario si desea clonar los datos en un fichero de seguridad antes de proceder al borrado.
        /// </summary>
        public void RemoveAll()
        {
            if (MessageBox.Show("Esta opción eliminará todos los registros de la base de datos\r\n\r\nEstá seguro de continuar ?", "Eliminar", MessageBoxButtons.YesNo ) == DialogResult.Yes)
            {
                if (MessageBox.Show("Desea crear una copia de seguridad de los datos antes de eliminar ?\r\n", "Clonar", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    cloneDataBase();
                }
                bTree = new SortedDictionary<string, docentes>();
                closeDatabase();
            }
            
        }



        /// <summary>
        /// Abra el fichero binario de datos y lo sube a la memoria por deserealización para convertir la trama de bytes a
        /// un tipo SortedDictionary<string, docentes> contenedor de registros ordenados.
        /// </summary>
        /// <returns>
        /// true    La apertura de datos ha finalizado correctamente
        /// false   Se ha producido un error
        /// </returns>
        public bool openDatabase()
        {
            string filename     = appPath + @"\" + dbName;
            bool ret            = true;

            if (File.Exists(filename))
            {
                incidencia = "ok";
                try
                {
                    using (FileStream bStream = new FileStream(filename, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite))
                    {
                        /* Deserializamos el stream de bytes del fichero binario para cargarlos en memoria */
                        bStream.Seek(0, SeekOrigin.Begin);
                        BinaryFormatter binary = new BinaryFormatter();
                        bTree = (SortedDictionary<string, docentes>)binary.Deserialize(bStream);
                        bStream.Close();
                    }
                }
                catch (Exception e)
                {
                    ret = false;
                    incidencia = e.Message;
                }
            }

            return ret;
        }

        /// <summary>
        /// Guarda los datos en memoria y los graba en el fichero binario transformando en un stream de bytes los
        /// registros contenidos en SortedDictionary<string, docentes>.
        /// </summary>
        /// <returns></returns>
        public bool closeDatabase()
        {
            string filename = appPath + @"\" + dbName;
            bool ret = true;
            incidencia = "ok";

            if (File.Exists(filename))
            {
                try
                {
                    using (FileStream bStream = new FileStream(filename, FileMode.Truncate, FileAccess.ReadWrite, FileShare.ReadWrite))
                    {
                        /* Serializamos los datos para grabarlos en formato binarioi en el fichero */
                        BinaryFormatter binary = new BinaryFormatter();
                        binary.Serialize(bStream, bTree);
                        bStream.Close();
                    }
                }
                catch (Exception e)
                {
                    ret = false;
                    incidencia = e.Message;
                }
            }
            else
            {
                try
                {
                    using (FileStream bStream = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite))
                    {
                        BinaryFormatter binary = new BinaryFormatter();
                        binary.Serialize(bStream, bTree);
                        bStream.Close();
                    }
                }
                catch (Exception e)
                {
                    ret = false;
                    incidencia = e.Message;
                }
            }

            return ret;
        }

        /// <summary>
        /// Clona la base de datos con el contenido en memoria y con diferente nombre del original
        /// </summary>
        /// <returns></returns>
        public bool cloneDataBase()
        {
            string filename = appPath + @"\clone_" + dbName;
            bool ret = true;
            incidencia = "ok";

            if (File.Exists(filename))
            {
                try
                {
                    using (FileStream bStream = new FileStream(filename, FileMode.Truncate, FileAccess.ReadWrite, FileShare.ReadWrite))
                    {
                        BinaryFormatter binary = new BinaryFormatter();
                        binary.Serialize(bStream, bTree);
                        bStream.Close();
                    }
                }
                catch (Exception e)
                {
                    ret = false;
                    incidencia = e.Message;
                }
            }
            else
            {
                try
                {
                    using (FileStream bStream = new FileStream(filename, FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite))
                    {
                        BinaryFormatter binary = new BinaryFormatter();
                        binary.Serialize(bStream, bTree);
                        bStream.Close();
                    }
                }
                catch (Exception e)
                {
                    ret = false;
                    incidencia = e.Message;
                }
            }

            return ret;
        }
    }
}
